<?php
/**
 * Plugin Name: CSS Optimizer - Remove Unused CSS
 * Plugin URI: https://zippisite.com
 * Description: Cleans up and removes unused CSS from your website. Also generates Critical CSS to improve PageSpeed Score.
 * Version: 1.5
 * Author: ZippiSite
 * Text Domain: css_optimizer
 * License: GPLv2
 */

// add wordpress options and menu
function cssoptimizer_settings() {
    register_setting('cssoptimizer_options', 'cssoptimizer_options_enable');
    register_setting('cssoptimizer_options', 'cssoptimizer_options_token');
    register_setting('cssoptimizer_options', 'cssoptimizer_options_generateccss');
    register_setting('cssoptimizer_options', 'cssoptimizer_options_safelist');
    register_setting('cssoptimizer_options', 'cssoptimizer_options_limitpagepost');
    register_setting('cssoptimizer_options', 'cssoptimizer_options_exclude');
    register_setting('cssoptimizer_options', 'cssoptimizer_options_exclude_css');
    register_setting('cssoptimizer_options', 'cssoptimizer_cache_clean');

    if ( is_admin() && get_option( 'cssoptimizer_cache_clean' ) ) {
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Begin clear cache.');
        if (is_plugin_active( 'autoptimize/autoptimize.php' ))
            autoptimizeCache::clearall();

        if (is_plugin_active( 'wp-rocket/wp-rocket.php' )) {
            // Clear all caching files.
            rocket_clean_domain();

            $rocket_path = WP_ROCKET_MINIFY_CACHE_PATH . '1/';
            $rocket_files = glob($rocket_path."/*.css");
            if (!empty($rocket_files)) {
                foreach ( $rocket_files as $file ) {
                    unlink( $file ); 
                }
            }
        }

        if (is_plugin_active( 'wp-fastest-cache/wpFastestCache.php' )) {
            do_action('wp_ajax_wpfc_delete_cache');

            $wpfc_path = WPFC_WP_CONTENT_DIR . '/cache/wpfc-minified/';
            $wpfc_files = glob($wpfc_path."/*.purge.css");
            if (!empty($wpfc_files)) {
                foreach ( $wpfc_files as $file ) {
                    unlink( $file ); 
                }
            }
        }

        if (is_plugin_active( 'sg-cachepress/sg-cachepress.php' )) {
            do_action('wp_ajax_admin_bar_purge_cache');
        }
    
        // for cache file without autoptimize
        $path = WP_CONTENT_DIR . '/cache/cssoptimizer/';
        $files = glob($path."/*.css");
        if (!empty($files)) {
            foreach ( $files as $file ) {
                unlink( $file ); 
            }
        }

        // clear log file
        file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_optimizer_log.txt', "Clear log...");
        cssop_write_log("");

        update_option( 'cssoptimizer_cache_clean', 0 );
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Finish clear cache.');

        update_option( 'cssoptimizer_first_time_guide', 0 );
    }

    if ( !file_exists( WP_CONTENT_DIR . '/cache/cssoptimizer/' ) ) {
        mkdir( WP_CONTENT_DIR . '/cache/cssoptimizer/', 0755, true );
    }

    if ( !file_exists( WP_CONTENT_DIR . '/cache/cssoptimizer/css_optimizer_log.txt' ) ) {
        cssop_write_log("");
    }

    if ( get_option('cssoptimizer_activation_redirect', false) ) {
        delete_option('cssoptimizer_activation_redirect');
        exit( wp_redirect("options-general.php?page=cssoptimizer") );
    }
}

// set default value when activating 
function cssoptimizer_activation(){
    do_action( 'cssoptimizer_default_options' );

    add_option('cssoptimizer_activation_redirect', true);

    if ( empty( get_option('cssoptimizer_first_time_guide') ) ) {
        add_option('cssoptimizer_first_time_guide', true);
    }

    if ( !get_option('cssoptimizer_first_time_guide') ) {
        update_option( 'cssoptimizer_first_time_guide', 1 );
    }
}

// add css file
function cssop_setting_up_css() {
    wp_register_style( 'cssoptimizer-styles', plugin_dir_url( __FILE__ ) . 'css/css_optimizer.css' );
    wp_enqueue_style( 'cssoptimizer-styles' );
}

// assign default value for option when installing plugin.
function cssoptimizer_default_values(){
    if(empty(get_option('cssoptimizer_options_safelist'))) {	
		update_option('cssoptimizer_options_safelist', 'slide|menu|nav|arrow|rs-|owl');
	}

    if(empty(get_option('cssoptimizer_options_exclude_css'))) {	
		update_option('cssoptimizer_options_exclude_css', 'wp-content/uploads/, admin-bar.min.css, dashicons.min.css');
	}
}

function cssoptimizer_add_settings_page() {
    add_options_page( __( 'Css Optimizer Options', 'cssoptimizer' ), 'CSS Optimizer', 'manage_options', 'cssoptimizer', 'cssoptimizer_settings_page' );
    add_submenu_page(
        null,
        __( 'Css Optimizer Job Queue', 'cssoptimizer' ),
        'CSS Optimizer',
        'manage_options',
        'cssoptimizer_jobqueue',
        'cssoptimizer_job_queue_page'
    );
    add_submenu_page(
        null,
        __( 'Css Optimizer Free Support', 'cssoptimizer' ),
        'CSS Optimizer',
        'manage_options',
        'cssoptimizer_support',
        'cssoptimizer_support_page'
    );
}

function cssoptimizer_settings_page() {
    ?>
    <div class="wrap">
    <h1><?php _e( 'CSS Optimizer', 'cssoptimizer_options' ); ?></h1>
    <?php echo wp_kses_post( cssop_show_first_time_guide() ); ?>
    <?php echo wp_kses_post( cssop_show_message_if_not_enabled() ); ?>
    <?php echo wp_kses_post( cssop_admin_tabs() ); ?>
    <form action="options.php" method="post">
    <?php settings_fields( 'cssoptimizer_options' ); ?>
    <h2 class="itemTitle"><?php _e( 'CSS Optimizer Options', 'cssoptimizer_options' ); ?></h2>
    <?php echo wp_kses_post( cssop_show_job_queue_status() ); ?>
    <table class="form-table">
    <tr valign="top">
    <th scope="row"><?php _e( 'Enable', 'cssoptimizer_options' ); ?></th>
    <td><input type="checkbox" id="cssoptimizer_options_enable" name="cssoptimizer_options_enable" <?php echo get_option( 'cssoptimizer_options_enable' ) ? 'checked="checked" ' : ''; ?>/>
    </tr>
    <tr valign="top">
    <th scope="row"><?php _e( 'Token', 'cssoptimizer_options' ); ?></th>
    <td><label><input type="text" style="width:100%;" name="cssoptimizer_options_token" value="<?php echo trim( esc_html( get_option( 'cssoptimizer_options_token' ) ) ) ; ?>"/></label>
    <?php echo wp_kses_post( cssop_validate_token( get_option( 'cssoptimizer_options_token' ) ) ) ; ?></td>
    </tr>
    <tr valign="top">
    <th scope="row"><?php _e( 'Generate Critical CSS', 'cssoptimizer_options' ); ?></th>
    <td><input type="checkbox" id="cssoptimizer_options_generateccss" name="cssoptimizer_options_generateccss" <?php echo get_option( 'cssoptimizer_options_generateccss' ) ? 'checked="checked" ' : ''; ?> />
    </tr>
    <tr valign="top">
    <th scope="row"><?php _e( 'Safelist', 'cssoptimizer_options' ); ?><?php _e( '</br><i><a href="https://zippisite.com/css-optimizer-safelist/">What is safelist?</a></i>' ); ?></th>
    <td><label><textarea rows="10" cols="10" style="width:100%;" name="cssoptimizer_options_safelist"><?php echo trim( esc_textarea( get_option( 'cssoptimizer_options_safelist' ) ) ) ; ?></textarea></label></td>
    </tr>
    <tr valign="top" id="limit_page_post">
    <th scope="row"><?php _e( 'Limit to pages / posts', 'cssoptimizer_options' ); ?><?php _e( '</br><i><a href="https://zippisite.com/css-optimizer-limit-to-exclude/">What is limit/exclude to pages/posts?</a></i>' ); ?></th>
    <td><label><textarea rows="10" id="cssoptimizer_options_limitpagepost" cols="10" style="width:100%;" name="cssoptimizer_options_limitpagepost"><?php echo trim( esc_textarea( get_option( 'cssoptimizer_options_limitpagepost' ) ) ) ; ?></textarea></label></td>
    </tr>
    <tr valign="top" class="exclude_page_post">
    <th scope="row"><?php _e( 'Exclude pages / posts', 'cssoptimizer_options' ); ?></th>
    <td><label><textarea rows="10" id="cssoptimizer_options_exclude" cols="10" style="width:100%;" name="cssoptimizer_options_exclude"><?php echo trim( esc_textarea( get_option( 'cssoptimizer_options_exclude' ) ) ) ; ?></textarea></label></td>
    </tr>
    <tr valign="top" class="exclude_css">
    <th scope="row"><?php _e( 'Exclude CSS from CSS Optimizer', 'cssoptimizer_options' ); ?></th>
    <td><label><textarea rows="10" id="cssoptimizer_options_exclude_css" cols="10" style="width:100%;" name="cssoptimizer_options_exclude_css"><?php echo trim( esc_textarea( get_option( 'cssoptimizer_options_exclude_css' ) ) ) ; ?></textarea></label></td>
    </tr>
    </table>
    <input type="submit" class="button button-primary" name="cssoptimizer_cache_clean" value="<?php _e( 'Save Changes', 'cssoptimizer_options' ); ?>" />
    </form>
    <br>
    <input type="button" onclick="location.href = '<?php echo esc_url(content_url() . '/cache/cssoptimizer/css_optimizer_log.txt'); ?>'" class="button button-secondary" value="<?php _e( 'View Logs', 'cssoptimizer_options' ); ?>" />
    </div>

    <script type="text/javascript">
        jQuery(document).ready(function() {
            if ( jQuery("#cssoptimizer_options_enable").is(':checked') ) {
                jQuery(".form-table").find('input, textarea, checkbox').attr("disabled", false);
            } else {
                jQuery(".form-table").find('input, textarea, checkbox').attr("disabled", true);
                jQuery("#cssoptimizer_options_enable").attr("disabled", false);
            }

            jQuery('#cssoptimizer_options_enable').change(function() {
                if(this.checked) {
                    jQuery(".form-table").find('input, textarea, checkbox').attr("disabled", false);

                    if ( jQuery.trim(jQuery( "#cssoptimizer_options_limitpagepost" ).val()).length > 0 ) { 
                        jQuery("#cssoptimizer_options_exclude").attr('disabled',true);
                    }

                    if ( jQuery.trim(jQuery( "#cssoptimizer_options_exclude" ).val()).length > 0 ) { 
                        jQuery("#cssoptimizer_options_limitpagepost").attr('disabled',true);
                    }

                } else {
                    jQuery(".form-table").find('input, textarea, checkbox').attr("disabled", true);
                    jQuery("#cssoptimizer_options_enable").attr("disabled", false);
                }
            });

            if ( jQuery.trim(jQuery( "#cssoptimizer_options_limitpagepost" ).val()).length > 0 ) { 
                jQuery("#exclude_page_post").fadeTo("fast",.33);
                jQuery("#cssoptimizer_options_exclude").attr('disabled',true);
            }

            if ( jQuery.trim(jQuery( "#cssoptimizer_options_exclude" ).val()).length > 0 ) { 
                jQuery("#limit_page_post").fadeTo("fast",.33);
                jQuery("#cssoptimizer_options_limitpagepost").attr('disabled',true);
            }

            jQuery( "#cssoptimizer_options_limitpagepost" ).blur(function() {
                if ( this.value.length > 0 ) {
                    jQuery("#exclude_page_post").fadeTo("fast",.33);
                    jQuery("#cssoptimizer_options_exclude").attr('disabled',true);
                } else {
                    jQuery("#exclude_page_post").fadeTo("fast",1);
                    jQuery("#cssoptimizer_options_exclude").attr('disabled',false);
                }
            });

            jQuery( "#cssoptimizer_options_exclude" ).blur(function() {
                if ( this.value.length > 0 ) { 
                    jQuery("#limit_page_post").fadeTo("fast",.33);
                    jQuery("#cssoptimizer_options_limitpagepost").attr('disabled',true);
                } else {
                    jQuery("#limit_page_post").fadeTo("fast",1);
                    jQuery("#cssoptimizer_options_limitpagepost").attr('disabled',false);
                }
            });

            jQuery( "form" ).submit(function() {
                jQuery(".form-table").find('input, textarea, checkbox').attr("disabled", false);                
            });
        });
    </script>
    <?php
}

function cssoptimizer_job_queue_page() {
    ?>
    <div class="wrap">
    <h1><?php _e( 'CSS Optimizer', 'cssoptimizer_options' ); ?></h1>
    <?php echo wp_kses_post( cssop_admin_tabs() ); ?>
    <h2 class="itemTitle">Refresh page to update Job Queue status. <?php _e( 'CSS Optimizer Job Queue', 'cssoptimizer_options' ); ?><?php _e( '<p><i><a href="https://zippisite.com/job-queue-tab/">What is Job Queue?</a></i></p>' ); ?></h2>
    <?php echo wp_kses_post(cssop_build_table(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt')) ; ?>
    </div>
    <?php
}

function cssoptimizer_support_page() {
    ?>
    <div class="wrap">
    <h1><?php _e( 'CSS Optimizer', 'cssoptimizer_options' ); ?></h1>
    <?php echo wp_kses_post( cssop_admin_tabs() ); ?>
    <h2 class="itemTitle"><?php _e( 'CSS Optimizer Free Support', 'cssoptimizer_options' ); ?></h2>
    <?php _e( '<p>We provide free support to help you use this plugin.</p><p> Make sure you read the quick start guide before using the plugin: <a href="https://zippisite.com/css-optimizer-installation-1-3/">https://zippisite.com/css-optimizer-installation-1-3/</a></p> <p>If you encounter any problem, or simply want us to help you setup this plugin, please e-mail us at <a href="mailto:cssopsupport@panorazzi.com">cssopsupport@panorazzi.com</a>.</p>', 'cssoptimizer_options' ); ?>
    </div>
    <?php
}

function cssop_admin_tabs() {
    $tabs        = array( 'cssoptimizer' => __( 'Options', 'cssoptimizer' ),  'cssoptimizer_jobqueue' => __( 'Job Queue', 'cssoptimizer' ),  'cssoptimizer_support' => __( 'Free Support', 'cssoptimizer' ) );
    $tab_content = '';
    $tabs_count  = count( $tabs );
    if ( $tabs_count > 1 ) {
        if ( isset( $_GET['page'] ) ) {
            $current_id = $_GET['page'];
        } else {
            $current_id = 'cssoptimizer';
        }
        $tab_content .= '<h2 class="nav-tab-wrapper">';
        foreach ( $tabs as $tab_id => $tab_name ) {
            if ( $current_id == $tab_id ) {
                $class = ' nav-tab-active';
            } else {
                $class = '';
            }
            $tab_content .= '<a class="nav-tab' . $class . '" href="?page=' . $tab_id . '">' . $tab_name . '</a>';
        }
        $tab_content .= '</h2>';
    } else {
        $tab_content = '<hr/>';
    }

    return $tab_content;
}

function cssop_show_job_queue_status() {
    $job_array_str = file_get_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt');
    $job_array = json_decode($job_array_str, true);
    $pending_job = 0;
    foreach($job_array as $key=>$value) {
        if($value['status'] === 'Pending' ){
            $pending_job++;
        }
    }

    if ( $pending_job > 1 ) {
        ?>
        <div class="job-queue-status"><p>
        <?php
        _e( $pending_job . " <a href=\"options-general.php?page=cssoptimizer_jobqueue\">jobs</a> are pending to be optimized." );
        ?>
        </p></div>
        <?php
    } elseif ( $pending_job == 1 ) {
        ?>
        <div class="job-queue-status"><p>
        <?php
        _e( $pending_job . " <a href=\"options-general.php?page=cssoptimizer_jobqueue\">jobs</a> is pending to be optimized." );
        ?>
        </p></div>
        <?php
    }
}

function cssop_show_first_time_guide() {
    if ( get_option('cssoptimizer_first_time_guide') ) {
        ?>
        <div class="notice notice-info is-dismissible" data-dismissible="cssop_guide_dismissible"><p>
        <?php
        _e( 'Guide for first time user: <a href=\"https://zippisite.com/css-optimizer-installation-1-3/\">https://zippisite.com/css-optimizer-installation-1-3/</a>', 'cssoptimizer' );
        ?>
        </p></div>
        <?php
    }
}

function cssop_show_message_if_not_enabled() {
    if ( !get_option('cssoptimizer_options_enable') ) {
        ?>
        <div class="notice-warning notice"><p>
        <?php
        _e( 'CSS Optimizer is not enabled, thus "Reduce Unused CSS" is not working. Please check "Enable" to turn on the "Reduce Unused CSS" feature.', 'cssoptimizer' );
        ?>
        </p></div>
        <?php
    }
}

function cssop_validate_token( $input ) {
    $newinput = trim( $input );

    if (empty($input)) {
        return '<a href="https://dashboard.zippisite.com/customer/sites">Get Authentication Token</a>';
    }

    if (!preg_match('/^[a-z0-9]{32}$/i', $newinput)) {
        //add_settings_error('cssoptimizer_options_token', 'cssoptimizer_options_token', __('Token Invalid - Wrong Authentication Token Format', 'cssoptimizer_options_token'));
        return '<p style="color: red;">Token Invalid - Wrong Authentication Token Format</p>';
    }

    $domain = get_option( 'home');
    $check_at_link = 'https://dashboard.zippisite.com/validate_auth_token?siteurl=' . $domain . '&authtoken=' . $input;
    $response = wp_remote_get( $check_at_link );
    $body = wp_remote_retrieve_body( $response );

    if ( is_wp_error( $response ) ) {
        $error_message = $response->get_error_message();
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error getting authentication token result: '. PHP_EOL . $error_message);
    }

    $data = json_decode($body);

    if($data) {
        if ($data->Success) {
            return '<p style="color: green;">Token Valid - Current usage for this site: ' . $data->SiteUsage . ' | <a href="https://dashboard.zippisite.com/customer/sites">View Full Usage</a></p>';
        }
        else {
            if ($data->Message == "Expired") {
                return '<p style="color: red;">Token Invalid - <a href="https://dashboard.zippisite.com/pricing-plan">Renew Plan Now</a></p>';
            }
            else if ($data->Message == "Overusage") {
                return '<p style="color: red;">Token Invalid - <a href="https://dashboard.zippisite.com/customer/sites">Current usage more then allowed usage per month</a></p>';
            }
            else {
                return '<p style="color: red;">Token Invalid - <a href="https://dashboard.zippisite.com/customer/sites">Get New Authentication Token</a></p>';
            }
        }
    } else {
        return '<p style="color: red;">Token Invalid - <a href="https://dashboard.zippisite.com/customer/sites">Get New Authentication Token</a></p>';
    }

    return '';
}

// create job queue table
function cssop_build_table($file_name){

    // start table
    $html = '<table class="cssop-list-table widefat striped">';
    // header row
    $html .= '<tr>';
    $html .= '<th>CSS Name</th>';
    $html .= '<th>Page URL</th>';
    $html .= '<th>Status</th>';
    $html .= '<th>Created On UTC</th>';
    $html .= '</tr>';
    
    if (file_exists($file_name) ) {
        $array = json_decode(file_get_contents($file_name), true);

        if (empty($array)) {
            $html .= '<tr><td colspan="4" style="text-align: center;">Empty record</td></tr>';
        } else {
            foreach( $array as $key=>$value){
                $html .= '<tr>';
                foreach($value as $key2=>$value2){
                    $add_html = '<td>' . htmlspecialchars($value2) . '</td>';

                    if ($key2 === 'run_in') {
                        // do nothing
                    }
                    else {
                        if ($key2 === 'created_on') {
                            $add_html = '<td>' . htmlspecialchars(date('m/d/Y h:i:s a', $value2)) . '</td>';
                        }

                        if ($key2 === 'css_name') {
                            if (strpos($value2, ', ') !== false){
                                $file_name_array = explode(', ', $value2);

                                if (count($file_name_array) === 1) {
                                    $add_html = '<td>' . htmlspecialchars($value2) . '</td>';
                                } elseif (count($file_name_array) === 2) {
                                    $add_html = '<td>' . htmlspecialchars($file_name_array[0]) . ' and ' . htmlspecialchars($file_name_array[1]) . '</td>';
                                } else {
                                    $add_html = '<td>' . htmlspecialchars($file_name_array[0]) . ' , ' . htmlspecialchars($file_name_array[1]) . ' and ' . htmlspecialchars(count($file_name_array) - 2) . ' other CSS files</td>';
                                }
                            }
                        }
            
                        $html .= $add_html;
                    }
                }
                $html .= '</tr>';
            }
        }    
    } else {
        $html .= '<tr><td colspan="4" style="text-align: center;">Empty record</td></tr>';
    }
    $html .= '</table>';
    return $html;
}

// settings link in plugins table
function cssoptimizer_action_links($actions, $plugin_file) {
	if(plugin_basename(__FILE__) == $plugin_file) {

        $settings_url = admin_url('options-general.php?page=cssoptimizer');

		$settings_link = array('settings' => '<a href="' . $settings_url . '">' . __('Settings', 'cssoptimizer') . '</a>');
		$actions = array_merge($settings_link, $actions);
	}
	return $actions;
}

add_action( 'cssoptimizer_default_options', 'cssoptimizer_default_values' );
add_action( 'admin_enqueue_scripts', 'cssop_setting_up_css' );
add_action( 'admin_menu', 'cssoptimizer_add_settings_page' );
add_action( 'admin_init', 'cssoptimizer_settings' );
add_filter('plugin_action_links', 'cssoptimizer_action_links', 10, 5);
register_activation_hook( __FILE__, 'cssoptimizer_activation' );



/* Utilities */

// Transform relative URL to full URL
function cssop_get_full_url ( $url, $post_url ) {
    $site_host = parse_url( $post_url, PHP_URL_HOST );

    if ( false === strpos( $url, $site_host ) ) {
        $url = get_home_url() . $url;
    } else {
        if ( is_ssl() ) {
            $url = 'https:' . $url;
        } else {
            $url = 'http:' . $url;
        }
    }

    return $url;
}

if (!function_exists('cssop_write_log')) {

    function cssop_write_log($log) {
        $cssop_log_file = WP_CONTENT_DIR . '/cache/cssoptimizer/css_optimizer_log.txt';

        if (is_array($log) || is_object($log)) {
            file_put_contents($cssop_log_file, print_r($log, true) . PHP_EOL, FILE_APPEND);
        } else {
            file_put_contents($cssop_log_file, $log . PHP_EOL, FILE_APPEND);
        }
    }
}

function cssop_download_Css($link) {
    $purged_link = 'http://api.zippisite.com/' . $link;

    $args = array(
        'timeout'     => 6000,
        'headers'     => array(
            'Content-Type'  => 'text/css',
        )
    );

    $response = wp_remote_get( $purged_link, $args );
    $purged_css = wp_remote_retrieve_body( $response );

    if ( is_wp_error( $response ) ) {
        $error_message = $response->get_error_message();
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error downloading Purged CSS file: '. PHP_EOL . $error_message);
        return false;
    }
    
    return $purged_css;
}

function cssop_add_criticalcss ( $content, $ccss_path ) {

    if ( file_exists( $ccss_path ) ) {
        $ccss_content = file_get_contents( $ccss_path );
        if ( ! empty( $ccss_content ) ) {
            $content_without_comment = preg_replace( '/<!--(.*)-->/Uis', '', $content );

            if (in_array( 'wp-rocket/wp-rocket.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && 1 === get_rocket_option( 'minify_css' ) ) {
                $selectedMatch = preg_match('/(<link[^>]*data-minify[^>]*\/>)/', $content_without_comment, $matches);
            }
            else {
                $selectedMatch = preg_match( '/(<link[^>]*all[^>]*\/>)/', $content_without_comment, $matches );
            }
                
            if ( $selectedMatch > 0 ) {
                $inline_ccss = $matches[0] . '<style id="cssop_ccss" media="all">' . $ccss_content . '</style>' ;
                $content = str_replace( $matches[0], $inline_ccss, $content );
            }

            $matchCount = preg_match_all( '/(<link[^>]*all[^>]*\/>)/', $content, $matches );

            if ( $matchCount > 0 ){
                foreach ($matches[0] as $match) {
                    $match_css = str_replace( "all", "print", $match );
                    $match_css = str_replace( "/>", "", $match_css);
                    $inline_ccss = $match_css . 'onload="this.onload=null;this.media=\'all\';" />' ;
                    $content = str_replace( $match, $inline_ccss, $content );
                }
            }
        }
        else {
            cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Critical CSS failed. Execution ended for ' . $post_url);
        }
    } 

    return $content;
}

// check authentication token
function cssop_check_token( ) { 
    $domain = get_option( 'home');
    $check_at_link = 'https://dashboard.zippisite.com/validate_auth_token?siteurl=' . $domain . '&authtoken=' . get_option('cssoptimizer_options_token');
    $response = wp_remote_get( $check_at_link );
    $body = wp_remote_retrieve_body( $response );

    if ( is_wp_error( $response ) ) {
        $error_message = $response->get_error_message();
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error getting authentication token result: '. PHP_EOL . $error_message);
        return false;
    }

    $data = json_decode($body);

    return $data->Success;
}

// Keep all job queue records for 24h. Remove all records that are more than 24h.
function cssop_filter_job_queue( ) {
    $job_array_str = file_get_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt');
    $job_array = json_decode($job_array_str, true);
    foreach($job_array as $key=>$value) {
        if(time() - 86400 >= $value['created_on']  ){
            unset($job_array[$key]);
        }

        if(time() - 120 >= $value['created_on'] && $value['status'] === 'Pending' ){
            $job_array[$key]['status'] = 'Cancelled';
        }
    }
    //$job_array = cssop_array_replace_status($job_array, str_replace( '.css', '.purge.css', basename($file) ), 'Running');
    $job_array_str = json_encode($job_array);
    file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt', $job_array_str);        
}

// Replace job queue record status for purge process.
function cssop_array_replace_status($file_name, $status) {
    $job_array_str = file_get_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt');
    $job_array = json_decode($job_array_str, true);

    if (!is_array($file_name)) {
        $file_css_name = $file_name;
        if (strpos($file_css_name, '.runque.css') !== false)
            $file_css_name = str_replace( '.runque.css', '.css', basename($file_name) );
        foreach($job_array as &$value) {
            if ($value['css_name'] === $file_css_name){
                $value['status'] = $status;
                break; // Stop the loop after we've found the item
            }
            unset($value);
        }
        $job_array_str = json_encode($job_array);
        file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt', $job_array_str);    
    } else {
        foreach($job_array as &$value) {
            $job_queue_file_names = explode(', ', $value['css_name']);
            sort($job_queue_file_names);
            sort($file_name);
            if($job_queue_file_names == $file_name ){
                $value['status'] = $status;
                break; // Stop the loop after we've found the item
            }
            unset($value);
        }
        //$job_array = cssop_array_replace_status($job_array, str_replace( '.css', '.purge.css', basename($file) ), 'Running');
        $job_array_str = json_encode($job_array);
        file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt', $job_array_str);    
    }
}

function cssop_clear_all_plugin_cache() {
    if (is_plugin_active( 'wp-rocket/wp-rocket.php' )) {
        rocket_clean_domain();
    }

    if (is_plugin_active( 'wp-fastest-cache/wpFastestCache.php' )) {
        do_action('wp_ajax_wpfc_delete_cache');
    }

    if (is_plugin_active( 'sg-cachepress/sg-cachepress.php' )) {
        do_action('wp_ajax_admin_bar_purge_cache');
    }
}

// add cron schedule
add_filter( 'cron_schedules', 'cssop_cron_interval' );

function cssop_cron_interval( $schedules ) { 
    $schedules['cssop_purge_job'] = array(
        'interval' => 60,
        'display'  => esc_html__( 'CSS Optimizer' ), );
    return $schedules;
}

// make sure cssoptimizer_autoptimize_queue is scheduled OK if cssop is enabled.
if ( get_option('cssoptimizer_options_enable') && cssop_check_token() && ! wp_next_scheduled( 'cssoptimizer_purge_queue' ) ) {
    wp_schedule_event( time(), 'cssop_purge_job', 'cssoptimizer_purge_queue');
}
add_action( 'cssoptimizer_purge_queue', 'cssop_noao_run_purgecss' );



/* CSS Optimizer Combine CSS */
function cssop_combine_css( $content, $css_name ) {
    $content_without_comments = preg_replace( '/<!--(.*)-->/Uis', '', $content );

    $regex = '/<link\s+([^>]+[\s\'"])?href\s*=\s*[\'"]\s*?([^\'"]+\.css(?:\?[^\'"]*)?)\s*?[\'"]([^>]+)?\/?>/';
    preg_match_all( $regex, $content_without_comments, $matches, PREG_SET_ORDER );

    if ( empty( $matches ) ) {        
        return '';
    }

    $data = array();

    // Loop through all styles in the queue and check for excludes.
    foreach ( $matches as $style ) {
        if ( false !== @strpos( $style[0], 'media=' ) && ! preg_match( '/media=["\'](?:\s*|[^"\']*?\b(all|screen)\b[^"\']*?)["\']/i', $style[0] ) ) {
            continue;
		}

		if ( false !== @strpos( $style[0], 'only screen and' ) ) {
            continue;
		}

        $host = parse_url( $style[2], PHP_URL_HOST );
        if ( strpos( get_home_url(null, '', 'https'), $host ) === false &&
        strpos( get_home_url(null, '', 'http'), $host ) === false &&
		!strpos( $style[2], 'wp-includes' ) ) {
			continue;
		}

        $exclude_css = array_filter( array_map( 'trim', explode( ',', get_option('cssoptimizer_options_exclude_css') ) ) ); 
        if ( is_array( $exclude_css ) && ! empty( $exclude_css ) ) {
            foreach ( $exclude_css as $match ) {
                if ( false !== strpos( $style, $match ) ) {
                    continue;
                }
            }
        }

        // Add the style url and tag.
        $data[ $style[2] ] = $style[0];
    }

    if ( empty( $data ) ) {
        return '';
    }

    foreach ( $data as $url => $tag ) {
        $content = str_replace( $tag, '', $content );
        $new_content[ $url ] = file_get_contents( $url );
    }

    $css_name = md5($css_name);

    $new_url   = WP_CONTENT_DIR . '/cache/cssoptimizer/cssop-combined-css-' . $css_name . '.css';

    $data = array(
        'handle' => 'cssop-combined-css-' . $css_name,
        'url'    => $new_url,
    );
    
    if ( !file_exists( $new_url ) ) {
        file_put_contents( $new_url, cssop_get_content_with_replacements( $new_content ));
    }

    return $new_url;
}

function cssop_get_content_with_replacements( $contents ) {
    // Set the new content var.
    $new_content = array();

    foreach ( $contents as $url => $content ) {
        $dir = trailingslashit( dirname( $url ) );

        $content = cssop_check_for_imports( $content, $url );
        // Remove source maps urls.
        $content = preg_replace(
            '~^(\/\/|\/\*)(#|@)\s(sourceURL|sourceMappingURL)=(.*)(\*\/)?$~m',
            '',
            $content
        );

        $regex = '/url\s*\(\s*(?!["\']?data:)(?![\'|\"]?[\#|\%|])([^)]+)\s*\)([^;},\s]*)/i';

        $replacements = array();

        preg_match_all( $regex, $content, $matches );

        if ( ! empty( $matches ) ) {
            foreach ( $matches[1] as $index => $match ) {
                $match = trim( $match, " \t\n\r\0\x0B\"'" );

                // Bail if the url is valid.
                if ( false == preg_match( '~(http(?:s)?:)?\/\/(?:[\w-]+\.)*([\w-]{1,63})(?:\.(?:\w{2,}))(?:$|\/)~', $match ) ) {
                    $replacement = str_replace( $match, $dir . $match, $matches[0][ $index ] );

                    $replacements[ $matches[0][ $index ] ] = $replacement;
                }
            }
        }

        $keys = array_map( 'strlen', array_keys( $replacements ) );
        array_multisort( $keys, SORT_DESC, $replacements );

        $new_content[] = str_replace( array_keys( $replacements ), array_values( $replacements ), $content );
    }

    return implode( "\n", $new_content );
}

function cssop_check_for_imports( $content, $url ) {
    // Get the file dir.
    $dir = trailingslashit( dirname( $url ) );
    // Check for imports in the style.
    preg_match_all( '/@import\s+["\'](.+?)["\']/i', $content, $matches );

    // Return the content if there are no matches.
    if ( empty( $matches ) ) {
        return $content;
    }

    // Loop through all matches and get the imported css.
    foreach ( $matches[1] as $match ) {
        $import_content = cssop_get_content_with_replacements(
            array(
                $url => file_get_contents( $dir . $match ),
            )
        );

        // Replace the @import with the css.
        $content = str_replace( $matches[0], $import_content, $content );
    }

    // Finally return the content.
    return $content;
}



/* Extract CSS Classes from JS */
function cssop_extract_css_from_js( $content ) {
    $content_without_comments = preg_replace( '/<!--(.*)-->/Uis', '', $content );

    $regex = '/<script\b([^>]*)>(?:\/\*\s*<!\[CDATA\[\s*\*\/)?\s*([\s\S]*?)\s*(?:\/\*\s*\]\]>\s*\*\/)?<\/script>/';
    preg_match_all( $regex, $content_without_comments, $scripts, PREG_SET_ORDER );

    if ( !empty( $scripts ) ) {        
        foreach ( $scripts as $script ) {
            preg_match( '/<script\s+([^>]+[\s\'"])?src\s*=\s*[\'"]\s*?([^\'"]+\.js(?:\?[^\'"]*)?)\s*?[\'"]([^>]+)?\/?>/', $script[0], $matches);

            // Add the style url and tag.
			if ( isset( $matches[2] ) ) {
                $is_external = false;

                $host = parse_url( $matches[2], PHP_URL_HOST );
                if ( strpos( get_home_url(null, '', 'https'), $host ) === false &&
                strpos( get_home_url(null, '', 'http'), $host ) === false &&
                !strpos( $matches[2], 'wp-includes' ) ) {
                    $is_external = true;
                }

                if ($is_external) {
                    // Try to fetch the file.
                    $request = wp_remote_get( $matches[2] );

                    // Bail if the request fails.
                    if ( is_wp_error( $request ) ) {
                        $js_content[ $script[0] ] = false;
                    }

                    if ( 200 !== wp_remote_retrieve_response_code( $request ) ) {
                        $js_content[ $script[0] ] = false;
                    }

                    // Get the file content from the request.
                    $file_content = wp_remote_retrieve_body( $request );
                } else {
                    $file_content = file_get_contents( $matches[2] );
                }
        
				$js_content[ $script[0] ] = $file_content;
			} else {
                preg_match( '/<script\b(?<attrs>[^>]*)>(?:\/\*\s*<!\[CDATA\[\s*\*\/)?\s*(?<content>[\s\S]*?)\s*(?:\/\*\s*\]\]>\s*\*\/)?<\/script>/', $script[0], $inline_matches );

                if ( false === strpos( $inline_matches['attrs'], 'src=' ) ) {
                    if ( strpos( $inline_matches['attrs'], 'type' ) !== false &&
                        ! preg_match( '/type\s*=\s*["\']?(?:text|application)\/(?:(?:x\-)?javascript|ecmascript)["\']?/i', $inline_matches['attrs'] ) ) {
                    
                        } else {
                            $js_content[ $script[0] ] = $inline_matches['content'];
                    }
                }
			}
        }

        $data = array_filter( $js_content );

        if ( !empty( $data ) ) {
            $saved_exclude_css = explode( '|', get_option('cssoptimizer_options_safelist') ); 
    
            foreach ( $data as $script => $js_content ) {
                preg_match_all( '/addClass\([ ]?[\'"]([-_a-zA-Z0-9]+)[\'"][ ]?\)/', $js_content, $css_name_matches, PREG_SET_ORDER );

                if (!empty($css_name_matches)) {
                    foreach ($css_name_matches as $css) {
                        if (!in_array( $css[1] ,$saved_exclude_css )) {
                            array_push($saved_exclude_css, $css[1]);
                        }
                    }
                }
            }

            if (!empty($saved_exclude_css)) {
                $new_exclude_css = implode('|', $saved_exclude_css);
                return $new_exclude_css;
            }
        }
    }

    return '';
}





/* Standard CSS Optimizer */

function cssop_noao_execute_purgecss( $buffer ) {
    if ( is_user_logged_in() ) {
        return $buffer;
    }

    $post_url = get_permalink();
    if (empty($post_url))
        return $buffer;

    if (empty($buffer)) 
        return $buffer;


    cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Begin execute CSS Optimizer on ' . $post_url);

    $enable = get_option('cssoptimizer_options_enable');
    if (!$enable){
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] CSS Optimizer is not activated. Execution ended on ' . $post_url);
        return $buffer;
    }

    // check post id or slug
    $exclude = get_option('cssoptimizer_options_exclude');
    if (!empty($exclude)){
        global $post;
        $post_slug = $post -> post_name;
        $post_id = get_the_ID();
        $exclude = str_replace(array("\r\n", "\n\r", "\r", "\n"), '|', $exclude);
        $exclude = strtolower(str_replace(' ', '', $exclude));
        $lines = explode('|', $exclude);
        foreach ($lines as $line) {
            if (strpos($line, $post_slug) !== false && !is_front_page()){
                cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Similar pages / posts slug (' . $post_slug . ') is found in "Exclude pages / posts" option. Execution ended for ' . $post_url);
                return $buffer;
            }
            else if ('id=' . $post_id == $line && !is_front_page()){
                cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Similar pages / posts ID (id =' . $post_id . ') is found in "Exclude pages / posts" option. Execution ended for ' . $post_url);
                return $buffer;
            }
            else if (strpos($line, '[home]') !== false) {
                if (is_front_page()){
                    cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Home page is found in "Exclude pages / posts" option. Execution ended for ' . $post_url);
                    return $buffer;
                }
            }
        }
    }    

    $limit = get_option('cssoptimizer_options_limitpagepost');
    if (!empty($limit)){
        $found = false;
        global $post;
        $post_slug = $post -> post_name;
        $post_id = get_the_ID();
        $limit = str_replace(array("\r\n", "\n\r", "\r", "\n"), '|', $limit);
        $limit = strtolower(str_replace(' ', '', $limit));
        $lines = explode('|', $limit);
        foreach ($lines as $line) {
            if (strpos($line, $post_slug) !== false && !is_front_page()){
                $found = true;
            }
            else if ('id=' . $post_id == $line && !is_front_page()){
                $found = true;
            }
            else if (strpos($line, '[home]') !== false) {
                if (is_front_page()){
                    $found = true;
                }
            }
        }

        if (!$found){
            cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] This page / post is not included in "Limit to pages / posts" option. Execution ended for ' . $post_url);
            return $buffer;
        }
    }    

    // Get <style> and <link>
    $css_url = array();
    $home_url = get_home_url();
    $home_url = str_replace( 'http://', '', $home_url );
    $home_url = str_replace( 'https://', '', $home_url );
    $buffer_without_comment = preg_replace( '/<!--(.*)-->/Uis', '', $buffer );
    if ( preg_match_all( '#<link[^>]*stylesheet[^>]*>#Usmi', $buffer_without_comment, $matches ) ) {
        foreach ( $matches[0] as $tag ) {
            $run_save_css = true;
            $exclude_css = array_filter( array_map( 'trim', explode( ',', get_option('cssoptimizer_options_exclude_css') ) ) ); //'wp-content/plugins/, wp-content/cache/, wp-content/uploads/, admin-bar.min.css, dashicons.min.css'
            if ( is_array( $exclude_css ) && ! empty( $exclude_css ) ) {
                foreach ( $exclude_css as $match ) {
                    if ( false !== strpos( $tag, $match ) ) {
                        $run_save_css = false;
                    }
                }
            }

            if ( preg_match( '#<link.*href=("|\')(.*)("|\')#Usmi', $tag, $source ) && $run_save_css ) {
                $url  = current( explode( '?', $source[2], 2 ) );

                if (strpos( $url, $home_url ) !== false && strpos( $url, '/cssoptimizer' ) === false) {
                    array_push($css_url, $url);
                }
            }
        }
    }
    
    if (empty($css_url)) {
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Empty css url. Execution ended for ' . $post_url);
        return $buffer;
    }

    // check authentication token first
    $domain = get_option( 'home');
    $check_at_link = 'https://dashboard.zippisite.com/validate_auth_token?siteurl=' . $domain . '&authtoken=' . get_option('cssoptimizer_options_token');
    $response = wp_remote_get( $check_at_link );
    $body = wp_remote_retrieve_body( $response );

    if ( is_wp_error( $response ) ) {
        $error_message = $response->get_error_message();
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error getting authentication token result: '. PHP_EOL . $error_message);
    }

    $data = json_decode($body);

    if ($data) {
        if (!$data->Success) {
            return $buffer;
        }
    }

    // check if there’s any file of the same name in css optimizer cache
    $run_purge_job = false;
    $cache_exist = false;
    $post_slug_name = md5($post_url);
    $job_queue_css_name = array();
    foreach ( $css_url as $url) {
        
        $full_url = $url;
        if ( filter_var($url, FILTER_VALIDATE_URL) === false )
            $full_url = cssop_get_full_url($url, $post_url);

        $css_old_path = str_replace( get_home_url(null, '', 'https') . '/', '', $full_url);
        $css_old_path = str_replace( get_home_url(null, '', 'http') . '/', '', $css_old_path);
        $css_new_name = str_replace( '/' , '_', $css_old_path);
        $css_new_name = $post_slug_name . '_' . $css_new_name;

        if (file_exists(WP_CONTENT_DIR . '/cache/cssoptimizer/' . $css_new_name)) {
            $buffer = str_replace( $url, content_url() . '/cache/cssoptimizer/' . $css_new_name, $buffer );
            $cache_exist = true;
        }
        else {

            $response = wp_remote_get( $full_url );
            if ( is_wp_error( $response ) ) {
                $error_message = $response->get_error_message();
                cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error getting css content from ' . $url . ' : '. PHP_EOL . $error_message);
            }
            else {
                $response_code = wp_remote_retrieve_response_code( $response );
                if ($response_code == 200)
                {
                    $body = wp_remote_retrieve_body( $response );
            
                    file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/' . $css_new_name, $body);
                    $buffer = str_replace( $url, content_url() . '/cache/cssoptimizer/' . $css_new_name, $buffer );
                    $run_purge_job = true;

                    file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/' . str_replace( '.css', '.runque.css', $css_new_name ), $post_url);        

                    array_push($job_queue_css_name, $css_new_name);
                } else {
                    cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Empty css content from ' . $url);
                }
            }
        }
    }

    if ($run_purge_job) {
        // save job
        $queue_name = implode(', ', $job_queue_css_name);

        $job_array_str = file_get_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt');
        $previous_job_array = json_decode($job_array_str, true);
        if (is_array($previous_job_array))
            $job_array = array_merge(array( array('css_name' => $queue_name, 'post_url' => $post_url, 'status' => 'Pending', 'run_in' => '-', 'created_on' => time())), $previous_job_array);
        else
            $job_array = array( array('css_name' => $queue_name, 'post_url' => $post_url, 'status' => 'Pending', 'run_in' => '-', 'created_on' => time()));
        $job_array_str = json_encode($job_array);
        file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/css_job_queue.txt', $job_array_str);

        // critical css
        if ( get_option('cssoptimizer_options_generateccss') ) {
            $new_combined_css = cssop_combine_css( $buffer, $post_url );
            if (empty($new_combined_css)) {
                cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Failed to combine css file. (' . $post_url . ')');
            }
        }
        
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Add Purge CSS job to queue. (' . $post_url . ')');
    } elseif ($cache_exist) {
        // critical css
        if ( get_option('cssoptimizer_options_generateccss') ) {
            $buffer = cssop_add_criticalcss( $buffer, WP_CONTENT_DIR . '/cache/cssoptimizer/cssop-combined-css-' . md5($post_url) . '.css' );
        }
        
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Cache exists. The purged css file is exist for this pages / posts. Execution ended for ' . $post_url);
    } else {
        // do nothing
    }

    return $buffer;
}

function cssop_noao_run_purgecss( ) {
    $path = WP_CONTENT_DIR . '/cache/cssoptimizer/';
    $files = glob($path."/*.runque.css");
    if (!empty($files)) {

        // select random file site url -> find same site url css runque file to purge.
        $same_site_files = array();
        $rand_file = $files[array_rand($files)];
        $rand_page_url = file_get_contents($rand_file);

        foreach ($files as $check_file) {
            $page_url = file_get_contents($check_file);
            if ($page_url === $rand_page_url) {
                array_push($same_site_files, $check_file);
            }
        }

        $css_url = array();
        $job_queue_css_name = array();
        foreach ($same_site_files as $file) {
            $css_name = str_replace( '.runque.css', '.css', basename($file) );

            array_push($css_url, content_url() . '/cache/cssoptimizer/' . $css_name);
            array_push($job_queue_css_name, $css_name);
        }

        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Begin execute Purge CSS on ' . $rand_page_url);
        // save running job
        cssop_array_replace_status($job_queue_css_name, 'Running');

        $html_content = file_get_contents($rand_page_url);
        $content_safelist = cssop_extract_css_from_js($html_content);

        $token = get_option('cssoptimizer_options_token');
        $safelist = $content_safelist;
        $safelist = str_replace(array("\r\n", "\n\r", "\r", "\n"), '|', $safelist);

        $purgeMethod = apply_filters('purge_css_method', 'purge');
        if ($purgeMethod === 'purge' || $purgeMethod === 'uncss') {  
            // do nothing
        } else {
            $purgeMethod = 'purge';
        }
        $purgeMethod = 'purge_1.3'; // panorazzi
        $post_url = "http://api.zippisite.com/$purgeMethod?pageUrl=" . urlencode($rand_page_url);

        $purgeSafelist = apply_filters('get_purge_safelist', $safelist);
        if (isset($purgeSafelist) && !empty($purgeSafelist)) {
            $post_url = $post_url . '&safelist=' . $purgeSafelist;
        }
    
        $purgeToken = apply_filters('get_purge_token', $token);
        if (isset($purgeToken) && !empty($purgeToken)) {
            $post_url = $post_url . "&token=$purgeToken";
        }
        
        $args = array(
            'method'      => 'POST',
            'timeout'     => 25000,
            'headers'     => array(
                'Content-Type'  => 'text/plain',
            ),
            'body'        => json_encode($css_url)
        );
    
        $response = wp_remote_post( $post_url, $args );
        $body = wp_remote_retrieve_body( $response );
        $data = json_decode($body);
    
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error getting result: '. PHP_EOL . $error_message);
        }

        $purged_files = explode(',', $data->purge_css);

        for ( $i = 0; $i < count($purged_files); $i++ ) {

            if (strpos($purged_files[$i], 'purged.css') !== false || strpos($purged_files[$i], 'uncss.css') !== false) {
                $purified_css = cssop_download_Css($purged_files[$i]);

                if ($purified_css) {
                    // replace the URL referenced in url() with an absolute URL in css
                    if ( preg_match_all( '#url\(("|\')(.*)("|\')\)#Usmi', $purified_css, $source ) ) {
                        foreach ($source[2] as $pic_url) {
                            if ( !preg_match( '/data:image/', $pic_url, $pic_matches ) ) {
                                $pic_root_url = str_replace( '_', '/', basename($css_url[$i]) );
                                $pic_root_url = str_replace( basename($pic_root_url), '', $pic_root_url);
                                $purified_css = str_replace( "\"" . $pic_url, "\"" . get_home_url() . '/' .$pic_root_url . $pic_url, $purified_css);
                                $purified_css = str_replace( "'" . $pic_url, "'" . get_home_url() . '/' .$pic_root_url . $pic_url, $purified_css);
                            }
                        }
                    }    

                   file_put_contents( WP_CONTENT_DIR . '/cache/cssoptimizer/' . basename($css_url[$i]) , $purified_css );
                }
            } else {
                if ($data->statusCode) {
                   cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Purge CSS fail for ' . $rand_page_url . ': Error ' .  $data->statusCode . ' - ' . $data->message );
                } else {
                   cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Purge CSS fail for ' . $rand_page_url . ': Error - ' . $data );
                }
            }
      
            wp_delete_file( WP_CONTENT_DIR . '/cache/cssoptimizer/' . str_replace( '.css', '.runque.css', basename($css_url[$i]) ) );
   
        }

        // generate critical css
        if (get_option('cssoptimizer_options_generateccss')) {
            $post_slug = md5($page_url);
        
            $combined_css = WP_CONTENT_URL . '/cache/cssoptimizer/cssop-combined-css-' . $post_slug . '.css';
            if (file_exists(WP_CONTENT_DIR . '/cache/cssoptimizer/cssop-combined-css-' . $post_slug . '.css')) {
                cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Begin execute Critical CSS on ' . $page_url);

                $ccss_post_url = "http://api.zippisite.com/criticalcss_1.5?cssUrl=" . urlencode($combined_css) . "&pageUrl=" . urlencode($rand_page_url);
                if (isset($purgeSafelist) && !empty($purgeSafelist)) {
                    $ccss_post_url = $ccss_post_url . '&safelist=' . $purgeSafelist;
                }

                if (isset($purgeToken) && !empty($purgeToken)) {
                    $ccss_post_url = $ccss_post_url . "&token=$purgeToken";
                }
        
                $css_args = array(
                    'method'      => 'POST',
                    'timeout'     => 25000,
                    'headers'     => array(
                        'Content-Type'  => 'text/plain',
                    )
                );

                $response = wp_remote_post( $ccss_post_url, $css_args );
                $body = wp_remote_retrieve_body( $response );
                $data = json_decode($body);

                if ( is_wp_error( $response ) ) {
                    $error_message = $response->get_error_message();
                    cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error getting result: '. PHP_EOL . $error_message);
                }        


                if ( strpos($data->critical_css, 'critical.css') !== false ) {
                    sleep(5);
                    $ccss = cssop_download_Css($data->critical_css);
           
                    if ($ccss) {
                        file_put_contents(WP_CONTENT_DIR . '/cache/cssoptimizer/cssop-combined-css-' . $post_slug . '.css', $ccss);
                    }
                } else {
                    if ($data->statusCode) {
                        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Critical CSS fail for ' . $page_url . ': Error ' .  $data->statusCode . ' - ' . $data->message );
                    } else {
                        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Critical CSS fail for ' . $page_url . ': Error - ' .  $data );
                    }
                }
    
                cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Finish execute Critical CSS on ' . $page_url);
            }
        }        
        
        // save running job
        cssop_array_replace_status( $job_queue_css_name, 'Completed');

        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Finish execute Purge CSS on ' . $rand_page_url);
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Finish execute CSS Optimizer on ' . $rand_page_url);

        cssop_clear_all_plugin_cache();
    }    

    cssop_filter_job_queue();
}

function cssop_template_redirect() {

    if ((in_array( 'autoptimize/autoptimize.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && 'on' === get_option( 'autoptimize_css', false )) ||
    (in_array( 'wp-rocket/wp-rocket.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && 1 === get_rocket_option( 'minify_css' )) ||
    (in_array( 'wp-fastest-cache/wpFastestCache.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && isset($GLOBALS["wp_fastest_cache_options"]->wpFastestCacheMinifyCss) ) ||
    ( in_array( 'sg-cachepress/sg-cachepress.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && 1 === (int) get_option( 'siteground_optimizer_optimize_css' ) ) ) {
        // do nothing
    } 
    else {
        ob_start();
        ob_start( 'cssop_noao_execute_purgecss' );
    }
}
add_action( 'template_redirect', 'cssop_template_redirect' );

function cssop_plugins_loaded() {
    if ( in_array( 'sg-cachepress/sg-cachepress.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && 
        1 === (int) get_option( 'siteground_optimizer_optimize_css' ) ) {
            ob_start( 'cssop_noao_execute_purgecss' );
    }
}
add_action( 'plugins_loaded' , 'cssop_plugins_loaded' );


// wooncherk

function cssop_test_connection() {
    cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Begin execute test connection for CSS Optimizer.');

    $test_link = 'http://api.zippisite.com/test_post';

    $args = array(
        'method'      => 'POST',
        'timeout'     => 45,
        'headers'     => array(
            'Content-Type'  => 'text/plain',
        )
    );
    
    $response = wp_remote_post( $test_link, $args );

    if ( is_wp_error( $response ) ) {
        $error_message = $response->get_error_message();
        cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Error getting test result: '. PHP_EOL . $error_message);
        return false;
    }
    
    cssop_write_log('[' . date('m/d/Y h:i:s a', time()) . '] Finish execute test connection for CSS Optimizer.');
}

add_filter( 'autoptimize_html_after_minify', 'cssop_noao_execute_purgecss', 10, 1 );

// wooncherk

add_filter( 'rocket_buffer', 'cssop_noao_execute_purgecss', 50, 1 );

add_filter( 'wpfc_buffer_callback_filter', 'cssop_noao_execute_purgecss', 10, 1 );
